package com.ym.mec.teacher.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.CourseHomeworkDao;
import com.ym.mec.biz.dal.dto.CourseHomeworkStudentDetailDto;
import com.ym.mec.biz.dal.dto.CourseHomeworkWrapper;
import com.ym.mec.biz.dal.dto.StudentLessonTrainingDetailWrapper;
import com.ym.mec.biz.dal.entity.CourseHomework;
import com.ym.mec.biz.dal.entity.ExtracurricularExercisesReply;
import com.ym.mec.biz.dal.entity.StudentCourseHomework;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.CourseHomeworkQueryInfo;
import com.ym.mec.biz.service.CourseHomeworkService;
import com.ym.mec.biz.service.ExtracurricularExercisesReplyService;
import com.ym.mec.biz.service.StudentCourseHomeworkService;
import com.ym.mec.biz.service.StudentLessonTrainingDetailService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Author Joburgess
 * @Date 2019/10/20
 */
@Api(tags = "作业服务")
@RequestMapping("courseHomework")
@RestController
public class CourseHomeworkController extends BaseController {

    @Autowired
    private CourseHomeworkService courseHomeworkService;
    @Autowired
    private StudentCourseHomeworkService studentCourseHomeworkService;
    @Autowired
    private ExtracurricularExercisesReplyService extracurricularExercisesReplyService;
    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private StudentLessonTrainingDetailService studentLessonTrainingDetailService;

    @Autowired
    private CourseHomeworkDao courseHomeworkDao;

    @ApiOperation(value = "布置课堂作业")
    @GetMapping("/addCourseHomework")
    @AuditLogAnnotation(operateName = "布置课堂作业")
    public HttpResponseResult addCourseHomework(CourseHomework courseHomework){
        courseHomeworkService.addCourseHomework(courseHomework);
        return succeed();
    }

    @ApiOperation(value = "根据班级获取教师布置的作业")
    @GetMapping("/findTeacherCourseHomeworkByClassGroup")
    public Object findTeacherCourseHomeworkByClassGroup(CourseHomeworkQueryInfo queryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if(Objects.isNull(sysUser)){
            return failed(HttpStatus.FORBIDDEN,"请登录");
        }
        queryInfo.setUserId(sysUser.getId().longValue());
        return succeed(courseHomeworkService.findTeacherCourseHomeworkByClassGroup(queryInfo));
    }

    @ApiOperation(value = "根据课程计划获取需要交作业的学生")
    @GetMapping("/findCourseStudents")
    public Object findCourseStudents(Long courseScheduleId,String userName){
        if (Objects.isNull(courseScheduleId)) {
            throw new BizException("请指定课程");
        }
        return succeed(studentCourseHomeworkService.findStudentCourseHomeworkByCourse(courseScheduleId, null,userName));
    }

    @ApiOperation(value = "根据课程计划获取需要交作业的学生-公用")
    @GetMapping("/findCourseStudentsPublic")
    public Object findCourseStudentsPublic(Long courseScheduleId,String userName, boolean extra){
        if(!extra){
            if (Objects.isNull(courseScheduleId)) {
                throw new BizException("请指定课程");
            }
            return succeed(studentCourseHomeworkService.findStudentCourseHomeworkByCourse(courseScheduleId, null,userName));
        }else{
            return succeed(extracurricularExercisesReplyService.findExtraExerciseStudents(courseScheduleId,userName));
        }
    }

    @ApiOperation(value = "获取学生作业界面详细信息")
    @GetMapping(value = "/findCourseHomeworkStudentDetail")
    public Object findCourseHomeworkStudentDetail(Long courseScheduleID,Long userId){
        StudentCourseHomework studentCourseHomework1 = studentCourseHomeworkService.get(courseScheduleID);
        if(Objects.isNull(studentCourseHomework1)){
            return failed("训练不存在");
        }
        CourseHomeworkStudentDetailDto courseHomeworkStudentDetail = studentCourseHomeworkService.findCourseHomeworkStudentDetail(studentCourseHomework1.getCourseScheduleId(), userId);
        if(Objects.nonNull(courseHomeworkStudentDetail)){
            StudentCourseHomework studentCourseHomework=new StudentCourseHomework();
            studentCourseHomework.setId(studentCourseHomework1.getId());
            studentCourseHomework.setIsView(YesOrNoEnum.YES);
            studentCourseHomeworkService.update(studentCourseHomework);
        }
        return succeed(courseHomeworkStudentDetail);
    }

    @ApiOperation(value = "获取学生作业界面详细信息-公用")
    @GetMapping(value = "/findCourseHomeworkStudentDetailPublic")
    public Object findCourseHomeworkStudentDetailPublic(Long courseScheduleID,Long userId, boolean extra){
        if(!extra){
            StudentCourseHomework studentCourseHomework1 = studentCourseHomeworkService.get(courseScheduleID);
            if(Objects.isNull(studentCourseHomework1)){
                return failed("训练不存在");
            }
            CourseHomeworkStudentDetailDto courseHomeworkStudentDetail = studentCourseHomeworkService.findCourseHomeworkStudentDetail(studentCourseHomework1.getCourseScheduleId(), userId);
            if(Objects.nonNull(courseHomeworkStudentDetail)&&Objects.nonNull(courseHomeworkStudentDetail.getAttachments())){
                StudentCourseHomework studentCourseHomework=new StudentCourseHomework();
                studentCourseHomework.setId(studentCourseHomework1.getId());
                studentCourseHomework.setIsView(YesOrNoEnum.YES);
                studentCourseHomeworkService.update(studentCourseHomework);
            }
            return succeed(courseHomeworkStudentDetail);
        }else{
            CourseHomeworkStudentDetailDto studentExtraExerciseDetail = extracurricularExercisesReplyService.findStudentExtraExerciseDetail(courseScheduleID);
            if(Objects.isNull(studentExtraExerciseDetail)){
                return failed("训练不存在");
            }
            if(Objects.nonNull(studentExtraExerciseDetail.getAttachments())&&studentExtraExerciseDetail.getStatus().equals(YesOrNoEnum.YES)){
                ExtracurricularExercisesReply extracurricularExercisesReply=new ExtracurricularExercisesReply();
                extracurricularExercisesReply.setId(studentExtraExerciseDetail.getCourseHomeworkId());
                extracurricularExercisesReply.setIsView(1);
                extracurricularExercisesReplyService.update(extracurricularExercisesReply);
            }
            return succeed(studentExtraExerciseDetail);
        }
    }



    @ApiOperation(value = "获取作业详情")
    @GetMapping(value = "/findCourseHomeworkDetail")
    public HttpResponseResult<CourseHomeworkWrapper.CourseHomeworkList> findCourseHomeworkDetail(Integer courseScheduleId,ELessonTrainingType type){
        if (ELessonTrainingType.HOMEWORK.equals(type)) {

            return succeed(courseHomeworkService.findCourseHomeworkDetail(courseScheduleId));
        } else {
            return succeed(courseHomeworkService.findCourseExtraHomeworkDetail(courseScheduleId));
        }
    }




    @ApiOperation(value = "根据课程计划获取需要交作业的学生-公用")
    @PostMapping("/findCourseStudentsPublic/v2")
    public HttpResponseResult<List<StudentCourseHomework>> findCourseStudentsPublicV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            return succeed(studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(query));
        }else{
            return succeed(studentCourseHomeworkService.findExtraExerciseStudentsV2(query));
        }
    }



    @ApiOperation(value = "根据课程计划获取需要交作业的学生统计")
    @PostMapping("/findCourseStudentsPublicSubject/v2")
    public HttpResponseResult<CourseHomeworkWrapper.StudentHomeworkRecordStat> findCourseStudentsPublicSubjectV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        List<StudentCourseHomework> list;
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            list =  (studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(query));
        }else{
            list =  (studentCourseHomeworkService.findExtraExerciseStudentsV2(query));
        }

        CourseHomeworkWrapper.StudentHomeworkRecordStat stat = new CourseHomeworkWrapper.StudentHomeworkRecordStat();
        stat.setAllNum(list.size());
        stat.setFinishNum((int) list.stream().filter(StudentCourseHomework::getFinishFlag).count());
        stat.setUnFinishNum(stat.getAllNum()-stat.getFinishNum());
        stat.setStudentCourseHomeworkList(list);
        return succeed(stat);

    }


    @ApiOperation(value = "老师练习详情的练习组内作业信息")
    @PostMapping("/findHomeworkGroupInfo")
    public HttpResponseResult<List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail>> findHomeworkGroupInfo(@Validated @RequestBody StudentLessonTrainingDetailWrapper.HomeworkGroupInfoQuery query){

        if (ELessonTrainingType.HOMEWORK.equals(query.getType())) {
            CourseHomework courseHomework = courseHomeworkDao.findByCourseSchedule(query.getCourseScheduleId().longValue());
            if(Objects.isNull(courseHomework)){
                throw new BizException("作业不存在");
            }
            query.setCourseScheduleId(courseHomework.getId().intValue());
        }
        Map<Integer, List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail>> groupMap = studentLessonTrainingDetailService.getGroupMap(query.getCourseScheduleId().longValue(), query.getType());


        return succeed(groupMap.get(query.getGroup()));

    }

    @ApiOperation(value = "查询单个学生的训练")
    @GetMapping("/findCourseHomeworkStudentDetailPublic/v2")
    public HttpResponseResult<StudentLessonTrainingDetailWrapper.StudentLessonTraining> findCourseHomeworkStudentDetailPublicV2(@RequestParam Long courseScheduleId, @RequestParam Long userId, @RequestParam ELessonTrainingType type){
        if(ELessonTrainingType.HOMEWORK.equals(type)){

            // 获取学生作业界面详细信息
            return succeed(studentCourseHomeworkService.findCourseHomeworkStudentDetailPublicV2(courseScheduleId, userId,type));
        }else{

            return succeed(studentCourseHomeworkService.findExtracurricularExercisesDetailPublicV2(courseScheduleId, userId, type));
        }
    }
}
